0001 * A>TYPE B:DEFLECT.SOL
0002 *
0003 *******************************************************
0004 **
0005 **       <<<  THE GAME OF DEFLECTION  >>>
0006 **                               BY ANDREW RECUPERO
0007 **
0008 ** ((( FROM KILOBAUD ISSUE # 14; FEB. 1978 )))
0009 **
0010 ** ---------------------------------------------------
0011 ** ( EXECUTE PROGRAM AT ADDRESS HEX 0050 )
0012 ** ---------------------------------------------------
0013 **
0014 **    TO START:  TYPE IN 0, 1, 2, OR 3 FOR ONE-OF-4 OPTION
0015 **                 INPUT 0 = 1 TARGET, CLEAR FIELD
0016 **                   "   1 = 1 TARGET, SHIFT DEFLECTORS
0017 **                   "   2 = 3 TARGETS, CLEAR FIELD
0018 **                   "   3 = 3 TARGETS, SHIFT DEFLECTORS
0019 **
0020 ** THE OBJECT IS TO HIT THE TARGET(S) BY DEFLECTING THE
0021 **      RUNNER TO THE TARGET.   DEFLECT RUNNER BY
0022 **      TYPING IN A '/' OR A ' '.   ALL TARGETS MUST BE
0023 **      HIT WITHIN A FIXED PERIOD OF TIME.
0024 **
0025 ** ....................................................
0026 ** LINES WITH  A  *@*  SYMBOL IN THE COMMENT FIELD MEAN
0027 **       THAT THE ADDRESS MAY BE DIFFERENT IN OTHER
0028 **       SYSTEMS.
0029 ** ....................................................
0030 **
0031 **
0032 ** (((  EQUATES  )))
0033 **
0034 STATS   EQU  0FAH    *@* KEYBOARD STATUS PORT
0035 STROB   EQU  01      *@* KEYBOARD STROBE BIT
0036 DATA    EQU  0FCH    * 'SOL' KEYBOARD DATA
0037 VDM     EQU  0CC00H  * 'SOL' BEGINNING OF SCREEN
0038 VDMPG   EQU  0CCH    * 'SOL' VDM RAM PAGE #
0039 VDMPT   EQU  0FEH    * 'SOL' VDM PORT ADDRESS
0040 TARGT   EQU  05H     * TARGET SYMBOL
0041 RUNNR   EQU  07H     * RUNNER SYMBOL
0042 USLSH   EQU  2FH     * UP SLASH CODE '/'
0043 DSLSH   EQU  5CH     * DOWN SLASH ' '
0044 LEFT    EQU  80H     * RUNNER
0045 DOWN    EQU  40H     *   DIRECTION CONTROL
0046 UP      EQU  20H    
0047 RIGHT   EQU  10H     *        EQUATES
0048 SP      EQU  06H     * STACK POINTER
0049 PSW     EQU  06H     * Program Status Word
0050 SOLOS   EQU  0C004H   *__* SOLOS return address
0051 *
0052 * - - - - - - - - - - - - - - - - - - - - - - - - - -
0053 *
0054 START   LXI  SP,STACK+8
0055         CALL CLEAR         * CLEAR SCREEN
0056 REIDL   CALL DRUNR         * DISPLAY RUNNER
0057 IDLE    CALL ACTIN         * MOVE RUNNER
0058         CALL DELAY         * EX. TIME DELAY & CHECK INPUT
0059         MOV  A,C           * KEYBD. INPUT TO ACCUMULATOR
0060         ORA  A             * TEST FOR NON-ZERO
0061         JZ   IDLE
0062         MVI  M,20H         * MOVE RUNNER
0063         RRC                * SHIFT DEFLECTORS ?
0064         PUSH PSW           * SAVE GAME OPTION IN A REG.
0065         JC   SHIFT         * ...JUMP TO SHIFT IF YES
0066         CALL CLEAR         * DON'T SHIFT, CLEAR SCREEN
0067         JMP  INIT
0068 SHIFT   LXI  B,VDM         * SHIFT
0069         MOV  D,B           *   SCREEN
0070         MOV  E,C           *     ONE
0071         INX  D             *       POSITION
0072 SMOR    LDAX D             *         TO
0073         STAX B             *           THE
0074         INX  B             *             LEFT
0075         INX  D
0076         MOV  A,D
0077         CPI  VDMPG+4       *@*
0078         JNZ  SMOR          * GO SHIFT MORE IF NOT DONE
0079 INIT    POP  PSW           * RESTORE GAME OPTION TO "A" REG
0080         MVI  C,1           * SET FOR ONE TARGET
0081         RRC                * GAME OPTION BIT 0 TO CARRY FLAG
0082         MVI  A,TARGT       *@* TARGET SYMBOL TO "A" REG.
0083         JNC  GAM1          * JUMP FOR 1 TARGET OPTION
0084         MVI  C,3           * INITIALIZE FOR 3 TARGETS
0085         STA  VDM+034CH     *@* PUT TARG LEFT OF CENTER
0086         STA  VDM+02F4H     *@* TARGET TO THE RIGHT
0087 GAM1    STA  VDM+0320H     *@* TARGET IN THE MIDDLE
0088         MOV  A,C
0089         STA  TARNM         * STORE # OF TARGETS
0090         LXI  H,01C0H       *@* INTIALIZE GAME TIMER
0091         SHLD TIMER         * SAVE TIMER VALUE
0092         CALL DRUNR         * START GAME-DISPLAY RUNNER
0093 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0094 *   THIS IS THE MAIN LOOP FOR THE GAME
0095 *
0096 MAINL   CALL DELAY         * WAIT AWHILE & CHK. KEYBD.
0097         CALL ACTIN
0098         PUSH H             * SAVE "HL"
0099         LHLD TIMER         * GET GAME TIMER
0100         DCX  H             * DECREMENT TIMER
0101         MOV  A,H           * CHECK IF GAME TIMER
0102         ORA  L             * ...EXPIRED
0103         SHLD TIMER         * STORE UPDATED TIMER
0104         POP  H             * RESTORE "HL"
0105         JNZ  MAINL         * JMP IF TIMER UNEXPIRED
0106         LXI  B,LOSE        * ...DISPLAY NO
0107         CALL SHOW          *   ...TIMER MESSAGE
0108         CALL WAER          * WAIT; THEN ERASE MSG.
0109         JMP  START         * TIMER EXPIRED, RESTART GAME
0110 *
0111 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0112 * THIS ROUTINE INITIALIZES AND DISPLAYS RUNNER
0113 *          "B" REGISTER HOLDS MOTION DIRECTION - CODED AS:
0114 *              REG. BIT  7 6 5 4 3 2 1 0
0115 *                        ! ! ! ! - - - -
0116 *                        ! ! ! RIGHT
0117 *                        ! ! UP
0118 *                        ! DOWN
0119 *                        LEFT
0120 *
0121 DRUNR   LXI  H,VDM+0040H   *@* RUNNER STARTING POSITION
0122         LXI  D,0100H       * X,Y RUNNER POSITION: D=VERT E=HORZ
0123         MVI  M,RUNNR       *@*  DISPLAY RUNNER
0124         MVI  B,RIGHT       * INITIALIZE MOTION 2 RIGHT
0125         RET
0126 *
0127 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0128 * THIS ROUTINE DOES ALL THE RUNNER MOVEMENT
0129 *
0130 ACTIN   MOV  A,M       * CHECK IF
0131         CPI  RUNNR     *@* .....RUNNER displayed
0132         JNZ  DF010
0133         MVI  M,20H      * REMOVE RUNNER FROM FIELD
0134 *
0135 * -=<*>=- *   CALCULATE NEW RUNNER POSITION
0136 *
0137 DF010   MOV  A,B
0138         CPI  RIGHT      * IS MOTION TO RIGHT ?
0139         JNZ  DF012      * ...JUMP IF NO
0140         INR  E          * MOVE ONE POSITION TO RIGHT
0141 DF012   CPI  UP         * IS MOTION UP ?
0142         JNZ  DF014      * ...JUMP IF NO
0143         DCR  D          * MOVE 1 POSITION UP
0144 DF014   CPI  DOWN       * IS MOTION DOWN ?
0145         JNZ  DF016      * ...JUMP IF NO
0146         INR  D          * MOVE 1 POSITION DOWN
0147 DF016   CPI  LEFT       * IS MOTION TO LEFT ?
0148         JNZ  DF018      * ...JUMP IF NO
0149         DCR  E          * MOVE 1 POSITION TO LEFT
0150 *
0151 * -=<*>=- *   CALCULATES FIELD POSITION FROM HORIZONTAL
0152 *             AND VERTICAL VALUES IN "D" AND "E" REG.;
0153 *             RESULT IN "H" AND "L".
0154 *
0155 DF018   MOV  A,D
0156         RRC
0157         RRC
0158         MOV  L,A
0159         ANI  03H
0160         ORI  VDMPG      *@*
0161         MOV  H,A        * SAVE HIGH PART OF ADDRESS
0162         MOV  A,L
0163         ANI  0C0H
0164         ADD  E
0165         MOV  L,A        * SAVE LOW PART OF ADDRESS
0166 *
0167         MOV  A,M        * CHECK NEXT FIELD POSITION
0168         CPI  TARGT      *@*  IS IT A TARGET ?
0169         JNZ  DF019      * JUMP IF IT DIDN'T HIT A TARGET
0170 *
0171 * -=<*>=- *      HIT  TARGET
0172 *
0173         MVI  M,20H      * REMOVE TARGET
0174         LDA  TARNM      * GET # OF TARGETS LEFT
0175         DCR  A          * REDUCE # BY ONE
0176         STA  TARNM      * STORE UPDATED TARGET COUNT
0177         RNZ             * RETURN IF MORE TARGETS
0178         POP  D          * ALL TARG GONE-FIX STACK P.
0179         LXI  B,WIN      * DISPLAY WIN MESSAGE
0180         CALL SHOW
0181         CALL WAER       * WAIT AWHILE; THEN ERASE MSG.
0182         JMP  REIDL      * GOTO IDLE LOOP
0183 *
0184 DF019   MOV  A,C        * PUT KEYBD. DATA INTO A REG.
0185         CPI  USLSH      * IS IT AN UP SLASH ? '/'
0186         JZ   DF020      * ...JUMP IF YES
0187         CPI  DSLSH      * IS IT A DOWN SLASH ? ' '
0188         JZ   DF020      * ...JMP 2 ANALYZE DFLEK IF GOOD IN
0189 *
0190 * -=<*>=- *    CHECK FOR RUNNER HITTING A DEFLECTOR
0191 *
0192 DFLCK   MOV  A,M
0193         CPI  20H      * IS NEXT POSITION BLANK ?
0194         JNZ  CHGMT    * DEFLECTOR IF NO-CHANGE DIRECTION
0195         CALL EDGCK    * REVERSE MOTION IF AT END OF FIELD
0196         MVI  M,RUNNR  *@*  DISPLAY RUNNER AT NEW LOCATION
0197         RET
0198 DF020   MOV  M,A      * INPUT WAS VALID: DISPLAY IT
0199 *
0200 * -=<*>=- *   CHANGE DIRECTION OF RUNNER MOTION
0201 *
0202 CHGMT   CPI  DSLSH    * IS IT A DOWN SLASH ?
0203         MOV  A,B      * DIRECTION FLAG TO REG.
0204         JNZ  UPSLH    * JUMP IF NOT DOWN SLASH
0205         ANI  0C0H     * IS MOTION LEFT OR DOWN ?
0206         MOV  A,B      * RESTORE DIRECTION FLAG TO "A" REG.
0207         JZ   DF030    * JUMP IF NOT LEFT OR DOWN
0208         RRC           * CHANGE DIRETION TO RIGHT OR UP
0209         RRC
0210         JMP  DF050
0211 DF030   RLC           * CHANGE DIRECTION TO LEFT OR DOWN
0212         RLC
0213         JMP  DF050
0214 UPSLH   ANI  0A0H     * IS MOTION TO LEFT OR UP ?
0215         MOV  A,B      * DIRECTION FLAG TO "A" REG.
0216         JZ   DF040    * JUMP IF NOT LEFT OR UP
0217         RRC           * CHANGE DIRECTION TO RIGHT OR DOWN
0218         JMP  DF050
0219 DF040   RLC           * CHANGE MOTION TO RIGHT OR DOWN
0220 DF050   MOV  B,A      * SAVE NEW DIRECTION
0221         CALL EDGCK    * IS RUNNER AT EDGE OF FIELD ?
0222         MOV  A,M      * GET DEFLECTOR IN CASE AT FIELD EDGE
0223         JZ   CHGMT    * IF AT FIELD EDGE-CHG DIRECTION AGAIN
0224         RET
0225 *
0226 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0227 * THIS ROUTINE CHECKS FOR EDGE OF FIELD, REVERSES MOTION
0228 *      AND SETS ZERO FLAG IF AT EDGE.
0229 *
0230 EDGCK   MOV   A,B     * LOAD MOTION DIRECTION FOR TESTING
0231         CPI   RIGHT   * IS MOTION TO RIGHT ?
0232         JNZ   DF060   * ...JUMP IF NO
0233         MVI   A,3FH   * FOR RIGHT EDGE HORZ='3F'
0234         CMP   E
0235         RNZ           * RETURN IF NOT AT RIGHT EDGE
0236         MVI   B,LEFT  * REVERSE DIRECTION TO LEFT
0237         RET
0238 DF060   CPI   UP      * CHECK FOR MOTION UP
0239         JNZ   DF070   * JUMP IF MOTION NOT UP
0240         XRA   A
0241         ORA   D       * CHECK FOR TOP - VERT=0
0242         RNZ           * RETURN IF NOT AT TOP
0243         MVI   B,DOWN  * REVERSE DIRECTION TO DOWN
0244         RET
0245 DF070   CPI   DOWN    * CHECK FOR MOTION DOWN
0246         JNZ   DF080   * ...JUMP IF MOTION NOT DOWN
0247         MVI   A,0FH   * CHECK FOR BOTTOM - VERT='0F'
0248         CMP   D
0249         RNZ           * RETURN IF NOT AT BOTTOM
0250         MVI   B,UP    * REVERSE DIRECTION TO UP
0251         RET
0252 DF080   XRA   A       * MUST BE MOTION TO LEFT
0253         ORA   E       * CK FOR LEFT EDGE - HORZ=0
0254         RNZ           * RETURN IF NOT LEFT EDGE
0255         MVI   B,RIGHT   * REVERSE DIRECTION TO RIGHT
0256         RET
0257 *
0258 * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0259 * THIS ROUTINE PERFORMS A TIME DELAY WHILE CONTINUOUSLY
0260 *   CHECKING FOR KEYBOARD INPUT - INPUT RETURN IN "C" REG.
0261 *
0262 DELAY   MVI  C,0      * INITIALIZE INPUT SAVE REG.
0263         PUSH D        * FREE UP "D" & "E" REGS.
0264 TDL     LXI  D,00900H  *LOAD DELAY VALUE
0265 DL01    DCX  D        * DECREMENT DELAY COUNT
0266         IN   STATS    *@* GET KEYBOARD STATUS
0267         ANI  STROB    *@* CHECK STROBE
0268         JNZ  DL02     *@* ...JUMP IF NO INPUT
0269         IN   DATA     *@* GET KEYBOARD DATA
0270         CPI  01BH      *__*   "ESC" key ?
0271         JZ   SOLOS     *__* ...yes, return to SOLOS
0272         ANI  7FH      *   REMOVE BIT 7
0273         MOV  C,A      *   SAVE INPUT IN "C" REG.
0274 DL02    MOV  A,D      *   CHECK FOR TIMER = 0
0275         ORA  E
0276         JNZ  DL01     
0277         POP  D        * DELAY OVER, RESTORE "D" & "E" REG.
0278         RET
0279 *
0280 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0281 * THIS ROUTINE INITIALIZES THE VDM PORT & CLEARS THE SCREEN
0282 *
0283 CLEAR   XRA  A
0284         OUT  VDMPT      *@* CLEAR VDM PORT
0285         LXI  H,VDM      *@* POINT TO START OF VDM RAM
0286         MVI  A,VDMPG+4  * "A" REG.=END OF VDM RAM
0287 CL01    MVI  M,20H      * BLANK LOCATION GIVEN BY "HL"
0288         INX  H
0289         CMP  H          * DONE ?
0290         JNZ  CL01       * ...JUMP IF NOT
0291         RET
0292 *
0293 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0294 * THIS ROUTINE DISPLAYS MESSAGE POINTED TO BY "BC" REG.
0295 *
0296 SHOW    LXI  D,VDM+031DH  *@* POINT TO DISPLAY LOCATION
0297         MVI  H,7          * MESSAGE LENGTH FIXED AT 7
0298 SH01    LDAX B            * GET BYTE OF MESSAGE
0299         STAX D            * DISPLAY IT
0300         INX  D            * BUMP TO NEXT BYTE
0301         INX  B
0302         DCR  H
0303         JNZ  SH01         * DONE ?
0304         RET
0305 *
0306 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
0307 * THIS ROUTINE WAITS AWHILE & THEN REMOVES MESSAGE
0308 *
0309 WAER     MVI  D,32       * LOAD UNITS OF DELAY
0310 WDLY     CALL DELAY      * WAIT ONE DELAY UNIT
0311          DCR  D          * CHECK IF DONE WAITING
0312          JNZ  WDLY
0313          LXI  B,BLNK     * POINT TO ALL BLANKS
0314          JMP  SHOW
0315 *
0316 * MESSAGES
0317 *
0318 WIN     ASC 'Winner '   * 'winner' MESSAGE
0319 LOSE    ASC 'NO TIME'   * 'NO TIME' mesage
0320 BLNK    ASC '        '
0321 *
0322 *  SAVE AREAS AND STACK
0323 *
0324 TIMER   DW 0       *TIMER SAVE AREA
0325 TARNM   DB 0       * # OF TARGETS SAVE AREA
0326 STACK   DS 8       * STACK AREA ( ONLY 4 DEEP )
0327 TDLAY   EQU  TDL+1 * game SPEED / TIME DELAY val.
0328 ENDIT   END
